from logging.config import dictConfig
from typing import Dict, Iterable, Optional

from cruft_helloworld.tools.dict_nested_update import nested_update


def config_loggers(
    default_logger_level: str = "INFO",
    default_logging_handler: str = "human_readable-colored-stdout",
    logging_config: Optional[Dict] = None,
    libs_names: Iterable[str] = (),
) -> None:
    default_loggers_libs_config = {
        "handlers": [default_logging_handler],
        "propagate": True,
        "level": default_logger_level,
    }

    default_logging_config = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "human_readable-colored": {
                "format": "%(asctime)s "
                "[%(bold)s%(name)s%(reset)s:%(threadName)s] "
                "%(log_color)s%(levelname)-8s%(reset)s "
                "%(message)s "
                "(%(bold)s%(filename)s%(reset)s:%(funcName)s:%(lineno)d)",
                "()": "colorlog.ColoredFormatter",
            },
        },
        "handlers": {
            "human_readable-colored-stdout": {
                "level": default_logger_level,
                "class": "colorlog.StreamHandler",
                "stream": "ext://sys.stdout",
                "formatter": "human_readable-colored",
                "filters": [],
            },
        },
        "loggers": {
            "root": {"handlers": [], "level": "NOTSET"},
            "cruft_helloworld": {
                "handlers": [default_logging_handler],
                "propagate": True,
                "level": default_logger_level,
            },
            **{lib_name: default_loggers_libs_config for lib_name in libs_names},
        },
    }

    dictConfig(nested_update(default_logging_config, logging_config or {}))



from enum import Enum


class IsoCodeContinentEmoji(Enum):
    """
    https://doc.bccnsoft.com/docs/php-docs-7-en/function.geoip-continent-code-by-name.html
    """

    AF = "globe_showing_europe-africa"  # Africa
    AN = "globe_with_meridians"  # Antarctica
    AS = "globe_showing_asia-australia"  # Asia
    EU = "globe_showing_europe-africa"  # Europe
    NA = "globe_showing_americas"  # North America
    OC = "globe_showing_asia-australia"  # Oceania
    SA = "globe_showing_americas"  # South America


class GlobeEmoji(Enum):
    """
    Declare enums for click option (`--globe-emoji`)
    and do (in the same time) a mapping between enum -> emoji name
    """

    EUROPE_AFRICA = "globe_showing_europe-africa"
    ASIA_AUSTRALIA = "globe_showing_asia-australia"
    AMERICAS = "globe_showing_americas"
    WITH_MERIDIANS = "globe_with_meridians"



import collections
from typing import Dict, Mapping, Union


def nested_update(dict_1: Dict, dict_2: Union[Dict, Mapping]):
    """
    >>> d1 = {"toto": {"tata": 42, "titi": 7}}
    >>> d2 = {"toto": {"titi": 42}, "abc": 11}
    >>> {**d1, **d2}    # classic dicts merge
    {'toto': {'titi': 42}, 'abc': 11}
    >>> nested_update(d1, d2)   # extended/nested dicts merge ("tata" is preserved)
    {'toto': {'tata': 42, 'titi': 42}, 'abc': 11}
    """
    # https://www.python.org/dev/peps/pep-0448/
    return {
        **dict_1,
        **{
            k: nested_update(dict_1.get(k, type(v)()), v)
            if isinstance(v, collections.abc.Mapping)
            else v
            for k, v in dict_2.items()
        },
    }



import logging
import re
from ipaddress import IPv4Address, ip_address
from typing import Optional

import requests
from geoip import IPInfo, geolite2

from cruft_helloworld.tools.enums import IsoCodeContinentEmoji

logger = logging.getLogger(__name__)


def get_external_ipv4(default_timeout: float = 1.0) -> Optional[IPv4Address]:
    """
    Use DuckDuckGo (external communication) for resolve external ip
    """
    # https://api.duckduckgo.com/api
    uri_to_ddg_api = "https://api.duckduckgo.com/?q=ip&format=json"
    external_ip = None
    try:
        # https://requests.readthedocs.io/en/master/user/quickstart/#timeouts
        raw = requests.get(uri_to_ddg_api, timeout=default_timeout)
        answer: str = raw.json().get("Answer", "")
        logger.debug("request(%s).Answer -> %s", uri_to_ddg_api, answer)
        # https://regex101.com/r/NMdWXw/1
        regex = (
            r"Your IP address is (?P<ip_address>[0-9]*\.[0-9]+\.[0-9]+\.[0-9]+) in.*"
        )
        match = re.match(regex, answer)
        if match is None:
            logger.error(f"Can't extract ip_address from: {answer}")
        else:
            # https://docs.python.org/3/library/ipaddress.html#ipaddress.ip_address
            external_ip = ip_address(match["ip_address"])
    except requests.exceptions.Timeout:
        logger.error(
            "Timeout (=%.5f) occurred on request: requests.get(%s)!",
            default_timeout,
            uri_to_ddg_api,
        )
    except requests.exceptions.RequestException:
        logger.error(
            "Can't perform internet request: requests.get(%s)!", uri_to_ddg_api
        )
    return external_ip


def find_globe_emoji_from_external_ip(
    default_iso_code_continent_emoji: IsoCodeContinentEmoji = IsoCodeContinentEmoji.EU,
) -> str:
    """
    Use external ip for finding appropriate globe emoji
    """
    external_ip = get_external_ipv4()
    if external_ip is None:
        logger.debug("No external IP found")
        return default_iso_code_continent_emoji.value
    match: IPInfo = geolite2.lookup(format(external_ip))
    return IsoCodeContinentEmoji[match.continent].value



import os
import sys

from pkg_resources import get_distribution

__version__ = get_distribution("cruft_helloworld").version

if sys.platform == "win32":
    os.environ["APPDATA"] = os.environ.get("APPDATA", "")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cruft_helloworld', 'cruft_helloworld.services', 'cruft_helloworld.tools']

package_data = \
{'': ['*']}

install_requires = \
['Cython>=0.29.21,<0.30.0',
 'click-default-group>=1.2.2,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'colorlog>=4.6.2,<5.0.0',
 'pyfiglet>=0.8.post1,<0.9',
 'python-geoip-geolite2>=2015.0303,<2016.0',
 'python-geoip-python3>=1.3,<2.0',
 'requests>=2.25.1,<3.0.0',
 'rich>=9.4.0,<10.0.0']

entry_points = \
{'console_scripts': ['helloworld = cruft_helloworld.app:cli']}

setup_kwargs = {
    'name': 'cruft-helloworld',
    'version': '0.4.2',
    'description': 'Cruft Python Hello-World',
    'long_description': '# cruft_helloworld\n\n![PyPI](https://img.shields.io/pypi/v/cruft_helloworld?style=flat-square)\n![GitHub Workflow Status (master)](https://img.shields.io/github/workflow/status/yoyonel/cruft_helloworld/Test%20&%20Lint/master?style=flat-square)\n![Coveralls github branch](https://img.shields.io/coveralls/github/yoyonel/cruft_helloworld/master?style=flat-square)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/cruft_helloworld?style=flat-square)\n![PyPI - License](https://img.shields.io/pypi/l/cruft_helloworld?style=flat-square)\n\nCruft Python Hello-World\n\n## Requirements\n\n* Python 3.7.3 or newer\n* [poetry](https://poetry.eustance.io/) 1.1 or newer\n\n## Installation\n\n```sh\npip install cruft_helloworld\n```\n\n## Development\n\nThis project uses [poetry](https://poetry.eustace.io/) for packaging and\nmanaging all dependencies and [pre-commit](https://pre-commit.com/) to run\n[flake8](http://flake8.pycqa.org/), [isort](https://pycqa.github.io/isort/),\n[mypy](http://mypy-lang.org/) and [black](https://github.com/python/black).\n\nClone this repository and run\n\n```bash\npoetry install\npoetry run pre-commit install\n```\n\nto create a virtual environment containing all dependencies.\nAfterwards, You can run the test suite using\n\n```bash\npoetry run pytest\n```\n\nThis repository follows the [Conventional Commits](https://www.conventionalcommits.org/)\nstyle.\n\n### Pycharm debugging\n[Debuggers and PyCharm](https://pytest-cov.readthedocs.io/en/latest/debuggers.html)\n> Coverage does not play well with other tracers simultaneously running.\n> This manifests itself in behaviour that PyCharm might not hit a breakpoint no matter what the user does.\n\n### Cookiecutter template\n\nThis project was created using [cruft](https://github.com/cruft/cruft) and the\n[cookiecutter-pyproject](https://github.com/escaped/cookiecutter-pypackage) template.\nIn order to update this repository to the latest template version run\n\n```sh\ncruft update\n```\n\nin the root of this repository.\n',
    'author': 'lionel atty',
    'author_email': 'yoyonel@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/yoyonel/cruft_helloworld',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.9,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python -*- coding: utf-8 -*-
"""
Usage: app.py [OPTIONS] COMMAND [ARGS]...

Options:
  --log-level [CRITICAL|FATAL|ERROR|WARN|WARNING|INFO|DEBUG|NOTSET]
                                  set logging level
  --help                          Show this message and exit.

Commands:
  hello-world*  Print a Hello-World message

---

Usage: app.py hello-world [OPTIONS]

Options:
  --globe-emoji [EUROPE_AFRICA|ASIA_AUSTRALIA|AMERICAS|WITH_MERIDIANS]
  --help                          Show this message and exit.
"""
import logging
from typing import Optional

import click
import pyfiglet
from click_default_group import DefaultGroup
from rich.console import Console

from cruft_helloworld import __version__
from cruft_helloworld.services.globe_emoji_with_geoip import (
    find_globe_emoji_from_external_ip,
)
from cruft_helloworld.tools.config_loggers import config_loggers
from cruft_helloworld.tools.enums import GlobeEmoji

console = Console()

logger = logging.getLogger(__name__)


PACKAGE_NAME = "cruft_helloworld"
PACKAGE_VERSION = __version__


# https://pypi.org/project/click-default-group/
# TODO: find a way to define log-level without removing default group command
@click.option(
    "--log-level",
    type=click.Choice(logging._nameToLevel.keys()),
    default="WARN",
    show_default=True,
    help="set logging level",
)
@click.option(
    "-v",
    "--verbose",
    # https://click.palletsprojects.com/en/7.x/options/#counting
    count=True,
)
@click.version_option(prog_name="cruft_helloworld")
@click.option("--show-banner", is_flag=True, help="Show a ASCII banner")
@click.group(
    cls=DefaultGroup,
    default="hello-world",
    default_if_no_args=True,
    invoke_without_command=True,
)
def cli(log_level, verbose, show_banner):
    if verbose:
        default_logger_level = "INFO" if verbose == 1 else "DEBUG"
    else:
        default_logger_level = getattr(logging, log_level.upper())
    config_loggers(default_logger_level=default_logger_level)
    if show_banner:
        # print the ASCII banner
        pyfiglet.print_figlet(f"{PACKAGE_NAME}, version {PACKAGE_VERSION}")


@cli.command(short_help="Print a Hello-World message")
@click.option(
    "--globe-emoji",
    required=False,
    default=None,
    # https://github.com/pallets/click/issues/605#issuecomment-667726724
    type=click.Choice(
        [e_global_emoji.name for e_global_emoji in GlobeEmoji], case_sensitive=False
    ),
    callback=lambda ctx, param, value: getattr(GlobeEmoji, value).value
    if value
    else None,
)
def hello_world(globe_emoji: Optional[str]):
    globe_emoji = globe_emoji or find_globe_emoji_from_external_ip()
    console.print(f"Hello :{globe_emoji}:")


# https://www.python.org/dev/peps/pep-0484/
cli.set_default_command(hello_world)  # type: ignore

if __name__ == "__main__":
    cli()
